@@ -2,31 +2,47 @@
 use strict;
 use warnings;
 our %gui;
-# OLD RDP: rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p
-our %prefs = (RDPCLIENT       => 'xfreerdp -g %Xx%Y --plugin cliprdr --plugin rdpsnd -T "%n - RemoteBox" %h:%p',
-              HEARTBEAT       => 1,
-              ADDADDITIONS    => 1,
-              RDPAUTOOPEN     => 1,
-              MSGLOGEXPAND    => 1,
-              GUESTLISTEXPAND => 1,
-              SSLVERIFY       => 0,
-              DEFRDPPORTS     => '3389-4389',
-              AUTOHINTDISP    => 0,
-              AUTOHINTDISPX   => 800,
-              AUTOHINTDISPY   => 600,
-              AUTOHINTDISPD   => 32,
-              STOPTYPE        => 'INSTANT');
+our %prefs = (RDPCLIENT         => 'xfreerdp /size:%Xx%Y /bpp:32 +clipboard /sound /t:"%n - RemoteBox" /v:%h:%p',
+              VNCCLIENT         => 'vncviewer -Shared -AcceptClipboard -SetPrimary -SendClipboard -SendPrimary -RemoteResize -DesktopSize %Xx%Y  %h:%p',
+              HEARTBEAT         => 1,
+              ADDADDITIONS      => 1,
+              RDPAUTOOPEN       => 1,
+              MSGLOGEXPAND      => 1,
+              GUESTLISTEXPAND   => 1,
+              AUTOSORTGUESTLIST => 1,
+              EXTENDEDDETAILS   => 0,
+              SSLVERIFY         => 0,
+              DEFRDPPORTS       => '3389-4389',
+              DEFVNCPORTS       => '5900-5999',
+              AUTOHINTDISP      => 0,
+              AUTOHINTDISPX     => 1280,
+              AUTOHINTDISPY     => 1024,
+              AUTOHINTDISPD     => 32,
+              STOPTYPE          => 'INSTANT',
+              EXPANDDETGEN      => 1,
+              EXPANDDETSYS      => 1,
+              EXPANDDETDISP     => 1,
+              EXPANDDETSTOR     => 1,
+              EXPANDDETAUDIO    => 1,
+              EXPANDDETNET      => 1,
+              EXPANDDETIO       => 1,
+              EXPANDDETUSB      => 1,
+              EXPANDDETSHARE    => 1,
+              EXPANDDETRUN      => 1,
+              EXPANDDETDESC     => 1,
+              AUTOCONNPROF      => '');
 
 # Import the current remotebox preferences
 sub rbprefs_get {
     $gui{PREFSDIR} = Glib::get_user_config_dir();
     $gui{CONFIGFILE} = "$gui{PREFSDIR}/remotebox.conf";
+    $gui{PROFILESFILE} = "$gui{PREFSDIR}/remotebox-profiles.conf";
     $gui{THUMBDIR} = "$gui{PREFSDIR}/remoteboxthumbs";
 
     # See if we can make some more useful defaults for OSs which don't have xfreerdp
     if ($^O =~ m/netbsd/i) { $prefs{RDPCLIENT} = 'rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'; }
     elsif ($^O =~ m/solaris/i) { $prefs{RDPCLIENT} = 'rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'; }
-    elsif ($^O =~ m/MSWin/) { $prefs{RDPCLIENT} = 'mstsc /w:%X /h:%Y /v:%h:%p >nul'; }
+    elsif ($^O =~ m/MSWin/) { $prefs{RDPCLIENT} = 'mstsc /w:%X /h:%Y /v:%h:%p'; }
 
     if (open(PREFS, '<', $gui{CONFIGFILE})) {
         my @contents = <PREFS>;
@@ -39,11 +55,28 @@ sub rbprefs_get {
         }
     }
     else { $prefs{URL}{qq[http://localhost:18083]} = 'URL'; } #Add a default URL
+
+    $gui{expanderMessages}->set_expanded($prefs{MSGLOGEXPAND});
+    $gui{checkbuttonShowDetails}->set_active($prefs{EXTENDEDDETAILS});
+
+    # Load in the connection profiles
+    if (open(PROFILES, '<', $gui{PROFILESFILE})) {
+        my @contents = <PROFILES>;
+        chomp(@contents);
+        close(PROFILES);
+        foreach (sort(@contents)) {
+            my ($pname, $url, $username, $password, $key) = split("\a", $_);
+            my @pass = split(' ', $password);
+            $password = pack("C*", @pass);
+            $password = &xor_pass($password, $key);
+            &addrow_profile(undef, $pname, $url, $username, $password);
+        }
+    }
 }
 
 # Save the current remotebox preferences
 sub rbprefs_save {
-	mkdir($gui{PREFSDIR}, 0755) unless (-e $gui{PREFSDIR});
+    mkdir($gui{PREFSDIR}, 0755) unless (-e $gui{PREFSDIR});
 
     if (open(PREFS, '>', $gui{CONFIGFILE})) {
         foreach my $key (keys %prefs) {
@@ -53,11 +86,53 @@ sub rbprefs_save {
             elsif ($key eq 'USER') {
                 foreach (keys %{$prefs{USER}}) { print PREFS "USER=$_\n"; }
             }
-            else { print PREFS "$key=$prefs{$key}\n"; }
+            else { print(PREFS "$key=$prefs{$key}\n"); }
         }
         close(PREFS);
     }
     else { warn "Unable to save preferences: $gui{CONFIGFILE}\n"; }
+
+
+}
+
+# Saves the profiles.
+# DO NOT call this from rbprefs_save otherwise there will be a race condition which will
+# empty the profiles file.
+sub rbprofiles_save {
+    # As this file might contain passwords, we'll create it 0600, but only if it's the first time we're creating it.
+    my $new = 1;
+    $new = 0 if (-e $gui{PROFILESFILE});
+
+    # Save the connection profiles
+    if (open(PROFILES, '>', $gui{PROFILESFILE})) {
+        chmod(0600, $gui{PROFILESFILE}) if ($new);
+        my $iter = $gui{liststoreProfiles}->get_iter_first();
+
+        while($iter) {
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 0) . "\a");
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 1) . "\a");
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 2) . "\a");
+            my $password = $gui{liststoreProfiles}->get_value($iter, 3);
+            my $key = &random_key(length($password));
+            $password = &xor_pass($password, $key);
+            my @ordpass = unpack("C*", $password);
+            $password = join(' ', @ordpass);
+            print(PROFILES "$password\a");
+            print(PROFILES "$key\n");
+            $iter = $gui{liststoreProfiles}->iter_next($iter);
+        }
+        close(PROFILES);
+    }
+}
+
+# Save RemoteBox's window position for restoration later
+sub save_window_pos {
+    my ($winname) = @_;
+    my $alloc = $gui{$winname}->allocation;
+    my ($w, $h) = ($alloc->width, $alloc->height);
+    my ($x,$y) = $gui{$winname}->get_position();
+    $prefs{"WINPOS_$winname"} = "$w:$h:$x:$y";
+    &rbprefs_save();
 }
 
 sub show_dialog_editnat {
@@ -67,14 +142,15 @@ sub show_dialog_editnat {
     $gui{checkbuttonvbprefsNATDHCP}->set_active(&bl(INATNetwork_getNeedDhcpServer($$natref{INATNetwork})));
     $gui{checkbuttonvbprefsNATipv6}->set_active(&bl(INATNetwork_getIPv6Enabled($$natref{INATNetwork})));
     $gui{checkbuttonvbprefsNATipv6route}->set_active(&bl(INATNetwork_getAdvertiseDefaultIPv6RouteEnabled($$natref{INATNetwork})));
-    &fill_list_pf($$natref{INATNetwork});
+    &fill_list_pf4($$natref{INATNetwork});
+    &fill_list_pf6($$natref{INATNetwork});
 
     do {
         my $response = $gui{dialogNATDetails}->run;
- 
+
         if ($response eq 'ok') {
             # Other entries do not require validation
-            if (!$gui{entryvbprefsNATName}->get_text()) { &show_err_msg('invalidname', '(Network Name)'); }
+            if (!$gui{entryvbprefsNATName}->get_text()) { &show_err_msg('invalidname'); }
             elsif (!&valid_cidr($gui{entryvbprefsNATCIDR}->get_text())) { &show_err_msg('invalidipv4cidr', '(Network CIDR)'); }
             else {
                 $gui{dialogNATDetails}->hide;
@@ -91,8 +167,8 @@ sub show_dialog_editnat {
     } until (!$gui{dialogNATDetails}->visible());
 }
 
-# Show dialog for adding a port forwarding rule
-sub show_dialog_pf {
+# Show dialog for adding an IPv4 port forwarding rule
+sub show_dialog_pf4 {
     $gui{entryPFIPv4Name}->set_text('Rule ' . int(rand(9999)));
 
     do {
@@ -100,21 +176,21 @@ sub show_dialog_pf {
 
         if ($response eq 'ok') {
             # No validation needed for other entries
-            if (!$gui{entryPFIPv4Name}->get_text()) { &show_err_msg('invalidname', '(Portforwarding rule name)'); }
+            if (!$gui{entryPFIPv4Name}->get_text()) { &show_err_msg('invalidname'); }
             elsif (!valid_ipv4($gui{entryPFIPv4HostIP}->get_text())) { &show_err_msg('invalidipv4address', '(Host IP)'); }
             elsif (!valid_ipv4($gui{entryPFIPv4GuestIP}->get_text())) { &show_err_msg('invalidipv4address', '(Guest IP)'); }
             else {
                 $gui{dialogPFIPv4}->hide;
                 my $natref = &getsel_list_vbprefsnat();
-                INATNetwork_addPortForwardRule($$natref{INATNetwork}, 0, 
+                INATNetwork_addPortForwardRule($$natref{INATNetwork}, 0,
                                        $gui{entryPFIPv4Name}->get_text(),
                                        &getsel_combo($gui{comboboxPFIPv4Protocol}, 1),
                                        $gui{entryPFIPv4HostIP}->get_text(),
-                                       int($gui{adjustmentPFIPv4HostPort}->get_value()),
+                                       int($gui{adjustmentPFIPHostPort}->get_value()),
                                        $gui{entryPFIPv4GuestIP}->get_text(),
-                                       int($gui{adjustmentPFIPv4GuestPort}->get_value()));
+                                       int($gui{adjustmentPFIPGuestPort}->get_value()));
 
-                &fill_list_pf($$natref{INATNetwork});
+                &fill_list_pf4($$natref{INATNetwork});
             }
         }
         else { $gui{dialogPFIPv4}->hide; }
@@ -122,20 +198,59 @@ sub show_dialog_pf {
     } until (!$gui{dialogPFIPv4}->visible());
 }
 
+# Show dialog for adding an IPv6 port forwarding rule
+sub show_dialog_pf6 {
+    $gui{entryPFIPv6Name}->set_text('Rule ' . int(rand(9999)));
+
+    do {
+        my $response = $gui{dialogPFIPv6}->run;
+
+        if ($response eq 'ok') {
+            # No validation needed for other entries
+            if (!$gui{entryPFIPv6Name}->get_text()) { &show_err_msg('invalidname', '(Portforwarding rule name)'); }
+            elsif (!valid_ipv6($gui{entryPFIPv6HostIP}->get_text())) { &show_err_msg('invalidipv6address', '(Host IP)'); }
+            elsif (!valid_ipv6($gui{entryPFIPv6GuestIP}->get_text())) { &show_err_msg('invalidip64address', '(Guest IP)'); }
+            else {
+                $gui{dialogPFIPv6}->hide;
+                my $natref = &getsel_list_vbprefsnat();
+                INATNetwork_addPortForwardRule($$natref{INATNetwork}, 1,
+                                       $gui{entryPFIPv6Name}->get_text(),
+                                       &getsel_combo($gui{comboboxPFIPv6Protocol}, 1),
+                                       $gui{entryPFIPv6HostIP}->get_text(),
+                                       int($gui{adjustmentPFIPHostPort}->get_value()),
+                                       $gui{entryPFIPv6GuestIP}->get_text(),
+                                       int($gui{adjustmentPFIPGuestPort}->get_value()));
+
+                &fill_list_pf6($$natref{INATNetwork});
+            }
+        }
+        else { $gui{dialogPFIPv6}->hide; }
+
+    } until (!$gui{dialogPFIPv6}->visible());
+}
+
 # Removes an IPv4 port forwarding rule to the selected NAT Network
-sub remove_pf_rule {
-    my $ruleref = &getsel_list_pf();
+sub remove_pf_rule4 {
+    my $ruleref = &getsel_list_pf4();
     INATNetwork_removePortForwardRule($$ruleref{INATNetwork}, 0, $$ruleref{Name});
-    &fill_list_pf($$ruleref{INATNetwork});
+    &fill_list_pf4($$ruleref{INATNetwork});
+}
+
+# Removes an IPv6 port forwarding rule to the selected NAT Network
+sub remove_pf_rule6 {
+    my $ruleref = &getsel_list_pf6();
+    INATNetwork_removePortForwardRule($$ruleref{INATNetwork}, 1, $$ruleref{Name});
+    &fill_list_pf6($$ruleref{INATNetwork});
 }
 
 # Shows the RemoteBox preferences dialog
 sub show_dialog_prefs {
     $gui{entryPrefsRDPClient}->set_text($prefs{RDPCLIENT});
-    $gui{entryPrefsRDPPorts}->set_text($prefs{DEFRDPPORTS});
+    $gui{entryPrefsVNCClient}->set_text($prefs{VNCCLIENT});
     $gui{checkbuttonPrefsRDPAuto}->set_active($prefs{RDPAUTOOPEN});
     $gui{checkbuttonPrefsHeartbeat}->set_active($prefs{HEARTBEAT});
     $gui{checkbuttonPrefsAddAdditions}->set_active($prefs{ADDADDITIONS});
+    $gui{checkbuttonPrefsAutoSortGuestList}->set_active($prefs{AUTOSORTGUESTLIST});
     $gui{checkbuttonPrefsAutoHint}->set_active($prefs{AUTOHINTDISP});
     $gui{spinbuttonPrefsWidth}->set_value($prefs{AUTOHINTDISPX});
     $gui{spinbuttonPrefsHeight}->set_value($prefs{AUTOHINTDISPY});
@@ -153,10 +268,11 @@ sub show_dialog_prefs {
 
     if ($response eq 'ok') {
         $prefs{RDPCLIENT} = $gui{entryPrefsRDPClient}->get_text();
-        $prefs{DEFRDPPORTS} = $gui{entryPrefsRDPPorts}->get_text();
+        $prefs{VNCCLIENT} = $gui{entryPrefsVNCClient}->get_text();
         $prefs{RDPAUTOOPEN} = int($gui{checkbuttonPrefsRDPAuto}->get_active());
         $prefs{HEARTBEAT} = int($gui{checkbuttonPrefsHeartbeat}->get_active());
         $prefs{ADDADDITIONS} = int($gui{checkbuttonPrefsAddAdditions}->get_active());
+        $prefs{AUTOSORTGUESTLIST} = int($gui{checkbuttonPrefsAutoSortGuestList}->get_active());
         $prefs{STOPTYPE} = &getsel_combo($gui{comboboxPrefsDefaultStop}, 1);
         $prefs{AUTOHINTDISP} = int($gui{checkbuttonPrefsAutoHint}->get_active());
         $prefs{AUTOHINTDISPX} = int($gui{spinbuttonPrefsWidth}->get_value());
@@ -166,45 +282,47 @@ sub show_dialog_prefs {
     }
 }
 
-# Clears the connection history so the connect dialog will be empty
-sub show_dialog_clearhistory() {
-    my $response = $gui{messagedialogClearHistory}->run;
-    $gui{messagedialogClearHistory}->hide;
-
-    if ($response eq 'ok') {
-        $prefs{USER} = ();
-        $prefs{URL} = ();
-        &rbprefs_save();
-        &addrow_log('Cleared connection history.');
-    }
-}
-
 # Shows the VirtualBox preferences dialog
-sub show_dialog_vbprefs() {
+sub show_dialog_vbprefs {
     my $vhost = &vhost();
     $gui{entryVBPrefsGenMachineFolder}->set_text($$vhost{machinedir});
     $gui{entryVBPrefsGenAutostartDBFolder}->set_text($$vhost{autostartdb});
     $gui{entryVBPrefsGenVRDPAuth}->set_text($$vhost{vrdelib});
     $gui{checkbuttonVBPrefsHWExclusive}->set_active(&bl($$vhost{hwexclusive}));
-    &fill_list_vbprefshon();
-    &fill_list_vbprefsnat();
-    $gui{dialogVBPrefs}->run();
+    my $response = $gui{dialogVBPrefs}->run();
     $gui{dialogVBPrefs}->hide();
 
-    my $machinedir = $gui{entryVBPrefsGenMachineFolder}->get_text();
-    my $autostartdir = $gui{entryVBPrefsGenAutostartDBFolder}->get_text();
-    my $vrdelib = $gui{entryVBPrefsGenVRDPAuth}->get_text();
-    $vrdelib =~ s/\.dll$//i;
-    $vrdelib =~ s/\.so$//i;
-    ISystemProperties_setDefaultMachineFolder($$vhost{ISystemProperties}, $machinedir) if ($machinedir ne $$vhost{machinedir});
-    ISystemProperties_setVRDEAuthLibrary($$vhost{ISystemProperties}, $vrdelib) if ($vrdelib ne $$vhost{vrdelib});
-    ISystemProperties_setAutostartDatabasePath($$vhost{ISystemProperties}, $autostartdir) if ($autostartdir ne $$vhost{autostartdb});
-    if ($gui{checkbuttonVBPrefsHWExclusive}->get_active()) { ISystemProperties_setExclusiveHwVirt($$vhost{ISystemProperties}, 'true'); }
-    else { ISystemProperties_setExclusiveHwVirt($$vhost{ISystemProperties}, 'false'); }
-    &clr_vhost(); # VB Prefs changes can potentially alter vhost values, so clear them to be repopulated
+    if ($response eq 'ok') {
+        my $machinedir = $gui{entryVBPrefsGenMachineFolder}->get_text();
+        my $autostartdir = $gui{entryVBPrefsGenAutostartDBFolder}->get_text();
+        my $vrdelib = $gui{entryVBPrefsGenVRDPAuth}->get_text();
+        $vrdelib =~ s/\.dll$//i;
+        $vrdelib =~ s/\.so$//i;
+        ISystemProperties_setDefaultMachineFolder($$vhost{ISystemProperties}, $machinedir) if ($machinedir ne $$vhost{machinedir});
+        ISystemProperties_setVRDEAuthLibrary($$vhost{ISystemProperties}, $vrdelib) if ($vrdelib ne $$vhost{vrdelib});
+        ISystemProperties_setAutostartDatabasePath($$vhost{ISystemProperties}, $autostartdir) if ($autostartdir ne $$vhost{autostartdb});
+        if ($gui{checkbuttonVBPrefsHWExclusive}->get_active()) { ISystemProperties_setExclusiveHwVirt($$vhost{ISystemProperties}, 'true'); }
+        else { ISystemProperties_setExclusiveHwVirt($$vhost{ISystemProperties}, 'false'); }
+        &clr_vhost(); # VB Prefs changes can potentially alter vhost values, so clear them to be repopulated
+    }
 }
 
-sub show_dialog_edithon() {
+# Shows the Host Network Manager dialog
+sub show_dialog_hostnetman {
+    &fill_list_vbprefshon();
+    &fill_list_vbprefsnat();
+    $gui{dialogHostNetMan}->run();
+    $gui{dialogHostNetMan}->hide();
+}
+
+# Shows the dialog for managing connection profiles
+sub show_dialog_profiles {
+    $gui{dialogProfiles}->run;
+    $gui{dialogProfiles}->hide;
+    &rbprofiles_save();
+}
+
+sub show_dialog_edithon {
     my $ifref = &getsel_list_vbprefshon();
     my @DHCPServers = IVirtualBox_getDHCPServers($gui{websn});
     my $IDHCPServer;
@@ -259,7 +377,7 @@ sub show_dialog_edithon() {
     } until (!$gui{dialogHON}->visible());
 }
 
-sub vbprefs_reset() {
+sub vbprefs_reset {
     my ($widget) = @_;
     my $ISystemProperties = IVirtualBox_getSystemProperties($gui{websn});
 
@@ -273,19 +391,19 @@ sub vbprefs_reset() {
     }
 }
 
-sub vbprefs_createhon() {
+sub vbprefs_createhon {
     my $IHost = IVirtualBox_getHost($gui{websn});
     my ($hostinterface, $IProgress) = IHost_createHostOnlyNetworkInterface($IHost);
-    &show_progress_window($IProgress, 'Adding host only network');
+    &show_progress_window($IProgress, 'Adding host only network', $gui{img}{ProgressNetwork});
     &fill_list_vbprefshon();
 }
 
-sub vbprefs_createnat() {
+sub vbprefs_createnat {
     IVirtualBox_createNATNetwork($gui{websn}, 'NatNetwork' . int(rand(9999)));
     &fill_list_vbprefsnat();
 }
 
-sub vbprefs_removehon() {
+sub vbprefs_removehon {
     my $ifref = &getsel_list_vbprefshon();
     my @DHCPServers = IVirtualBox_getDHCPServers($gui{websn});
 
@@ -298,18 +416,19 @@ sub vbprefs_removehon() {
     }
 
     my $IHost = IVirtualBox_getHost($gui{websn});
-    IHost_removeHostOnlyNetworkInterface($IHost, $$ifref{Uuid});
+    my $IProgress = IHost_removeHostOnlyNetworkInterface($IHost, $$ifref{Uuid});
+    &show_progress_window($IProgress, 'Removing host only network');
     &fill_list_vbprefshon();
 }
 
-sub vbprefs_removenat() {
+sub vbprefs_removenat {
     my $natref = &getsel_list_vbprefsnat();
     IVirtualBox_removeNATNetwork($gui{websn}, $$natref{INATNetwork});
     &fill_list_vbprefsnat();
 }
 
 # Can't use getsel_list_vbprefsnat in here due to the way the signals are propagated
-sub vbprefs_togglenat() {
+sub vbprefs_togglenat {
     my ($widget, $path_str, $model) = @_;
     my $iter = $model->get_iter(Gtk2::TreePath->new_from_string($path_str));
     my $val = $model->get($iter, 0);
@@ -318,25 +437,20 @@ sub vbprefs_togglenat() {
     &fill_list_vbprefsnat();
 }
 
-sub rbprefs_msglog() {
+sub rbprefs_msglog {
     my ($widget) = @_;
     $prefs{MSGLOGEXPAND} = int($widget->get_expanded()); # int forces 0 on undefined
     &rbprefs_save();
 }
 
-sub rbprefs_saveurluser() {
-    $prefs{SAVEURLUSER} = $gui{checkbuttonConnectSave}->get_active();
-    &rbprefs_save();
-}
-
-sub rbprefs_sslverify() {
+sub rbprefs_sslverify {
     my $active = $gui{checkbuttonConnectSSL}->get_active();
     $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = $active;
     $prefs{SSLVERIFY} = $active;
     &rbprefs_save();
 }
 
-sub rbprefs_autohint() {
+sub rbprefs_autohint {
     if ($gui{checkbuttonPrefsAutoHint}->get_active()) {
         $gui{spinbuttonPrefsWidth}->set_sensitive(1);
         $gui{spinbuttonPrefsHeight}->set_sensitive(1);
@@ -349,8 +463,34 @@ sub rbprefs_autohint() {
     }
 }
 
-sub vbprefs_hon_dhcp_toggle() {
+sub vbprefs_hon_dhcp_toggle {
     $gui{tableHONDHCP}->set_sensitive($gui{checkbuttonHONDHCP}->get_active()); # Ghost/Unghost other widgets based on dhcp enabled
 }
 
-1;
\ No newline at end of file
+# Shows the preset RDP options for selection
+sub show_rdppreset_menu {
+    my ($widget, $event) = @_; #$event->time
+    $gui{menuRDPPreset}->popup(undef, undef, undef, undef, 0, $event->time) if ($event->button == 1);
+    return 0;
+}
+
+# Shows the preset VNC options for selection
+sub show_vncpreset_menu {
+    my ($widget, $event) = @_; #$event->time
+    $gui{menuVNCPreset}->popup(undef, undef, undef, undef, 0, $event->time) if ($event->button == 1);
+    return 0;
+}
+
+# Updates the RDP widget with the selected preset
+sub set_rdppreset {
+    my ($widget, $command) = @_;
+    $gui{entryPrefsRDPClient}->set_text($command);
+}
+
+# Updates the VNC entry widget with the selected preset
+sub set_vncpreset {
+    my ($widget, $command) = @_;
+    $gui{entryPrefsVNCClient}->set_text($command);
+}
+
+1;
